@@ -4,8 +4,11 @@ import logging
 from pathlib import Path
 from PyPDF2 import PdfReader
 import dotenv
+import logging
+from inspect import signature
 
 dotenv.load_dotenv()
+logger = logging.getLogger(__name__)
 
 
 class BaseParser(ABC):
@@ -27,39 +30,62 @@ class Parser(BaseParser):
     """Factory + registry interface for all parsers."""
 
     @classmethod
-    def available_parsers(cls) -> dict[str, list[str]]:
-        """Return registered parsers and their init arguments."""
+    def available_parsers(cls) -> None:
+        """Print registered parsers and their init arguments."""
         import inspect
         result = {}
         for name, parser_cls in BaseParser._registry.items():
             sig = inspect.signature(parser_cls.__init__)
             result[name] = [p for p in sig.parameters if p != "self"]
         result.pop('', None)
-        return result
+        for key, values in result.items():
+            print(f"{key} with parameters: {values}")
 
     @classmethod
-    def create(cls, config: dict | None = None, **kwargs) -> BaseParser:
-        provider = kwargs["provider_and_model"].split(":")[0].lower()
-        model = kwargs["provider_and_model"].split(":")[1].lower()
+    def create(cls, config_or_dict: "Parameter | dict", **kwargs) -> BaseParser:
+        """
+        Dynamically create parser instances from a config object or dict.
+        Only passes arguments accepted by the parser constructor.
+        """
+        # Convert Parameter -> dict if needed
+        if hasattr(config_or_dict, "model_dump"):
+            config_dict = config_or_dict.model_dump()
+        elif isinstance(config_or_dict, dict):
+            config_dict = config_or_dict
+        else:
+            raise TypeError("Expected Parameter instance or dict for config_or_dict")
+
+        if "provider_and_model" not in kwargs and "provider_and_model" not in config_dict:
+            raise ValueError("provider_and_model must be specified")
+
+        # Merge dict + kwargs (kwargs take precedence)
+        merged_args = {**config_dict, **kwargs}
+
+        # Extract provider/model info
+        provider_and_model = merged_args.get("provider_and_model")
+        provider, model = provider_and_model.split(":")
+        provider = provider.lower()
+        model = model.lower()
+
         if provider not in BaseParser._registry:
-            raise ValueError(f"Unknown parser '{provider}'. "
-                             f"Available: {cls.available_parsers()}")
+            raise ValueError(f"Unknown parser '{provider}'. Available: {list(BaseParser._registry)}")
 
         parser_cls = BaseParser._registry[provider]
-        import inspect
-        merged_args = {**(config or {}), **kwargs}
-        if "model" in inspect.signature(parser_cls.__init__).parameters:
-            merged_args["model"] = model
 
-        # Remove keys not in constructor
-        sig = inspect.signature(parser_cls.__init__)
+        # Inspect constructor and only pass valid arguments
+        sig = signature(parser_cls.__init__)
         valid_args = {k: v for k, v in merged_args.items() if k in sig.parameters and k != "self"}
 
+        # Auto-insert 'model' if required
+        if "model" in sig.parameters and "model" not in valid_args:
+            valid_args["model"] = model
+
+        # Auto-insert 'provider_and_model' if required
+        if "provider_and_model" in sig.parameters and "provider_and_model" not in valid_args:
+            valid_args["provider_and_model"] = provider_and_model
+
         return parser_cls(**valid_args)
 
-
-
-
     @abstractmethod
     def parse(self, text: str):
         pass
@@ -107,10 +133,10 @@ class MistralOCRParser(BaseParser, name="mistralocr"):
 
 
 class LangChainParser(BaseParser, name="langchain"):
-    def __init__(self, model: str):
+    def __init__(self, provider_and_model: str):
         from langchain.llms import OpenAI
-        self.model_name = model
-        self.model = OpenAI(model_name=model)
+        self.model = provider_and_model.split(":")[1]
+        self.model = OpenAI(model_name=self.model)
 
     def parse(self, text: str) -> dict:
         response = self.model(text)
@@ -138,36 +164,37 @@ class LlamaParser(BaseParser, name="llamaparser"):
 
 
 class HuggingFaceParser(BaseParser, name="huggingface"):
-    def __init__(self, model: str, result_type: str) -> None:
-        from transformers import pipeline
-
-        if result_type.lower() in ("md", "markdown"):
-            self.result_type = "markdown"
-        else:
-            self.result_type = "text"
+    def __init__(self, provider_and_model: str) -> None:
+        from transformers import TrOCRProcessor, VisionEncoderDecoderModel
 
         api_key = os.getenv("HF-API-TOKEN")
         if not api_key:
             raise EnvironmentError("Missing HF-API-TOKEN in .env file.")
 
-        self.parser = pipeline(
-            task="document-question-answering",
-            model=model,
-            use_auth_token=api_key
-        )
+        model_name = provider_and_model.split(":")[1]
+        logger.info(f"Loading Hugging Face OCR model: {model_name}")
+
+        self.processor = TrOCRProcessor.from_pretrained(model_name, token=api_key)
+        self.model = VisionEncoderDecoderModel.from_pretrained(model_name, token=api_key)
+
+        logger.info("Model and processor loaded successfully.")
 
     def parse(self, file_path: Path) -> str:
-        import fitz
-        pdf_doc = fitz.open(file_path)
-        output_parts = []
+        from pdf2image import convert_from_path
+        logger.info(f"Converting PDF to images: {file_path}")
+        pages = convert_from_path(file_path, dpi=300)
+        logger.info(f"PDF conversion complete. Total pages: {len(pages)}")
 
-        for page in pdf_doc:
-            pix = page.get_pixmap(dpi=200)
-            img_bytes = pix.tobytes("png")
-            response = self.parser(img_bytes, question="Extract all text")
-            if response and "answer" in response[0]:
-                output_parts.append(response[0]["answer"])
+        all_text = ""
+        for i, page in enumerate(pages, start=1):
+            logger.info(f"Running OCR on page {i}/{len(pages)}")
+            pixel_values = self.processor(page, return_tensors="pt").pixel_values
+            generated_ids = self.model.generate(pixel_values)
+            text = self.processor.batch_decode(generated_ids, skip_special_tokens=True)[0]
+            all_text += text + "\n"
+            logger.debug(f"OCR text (page {i}): {text[:100]}...")  # preview first 100 chars
 
-        return "\n\n".join(output_parts) if self.result_type == "markdown" else " ".join(output_parts)
+        logger.info("OCR completed for all pages.")
+        return all_text
 
 
